# Makefile.tcl ---
# ------------------------------------------------------------------------
# (c) 2019, Johann Oberdorfer - Engineering Support | CAD | Software
#     johann.oberdorfer [at] gmail <dot> com
#     www.johann-oberdorfer.eu
# ------------------------------------------------------------------------
# This source file is distributed under the BSD license.
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#   See the BSD License for more details.
# ------------------------------------------------------------------------
#
# Purpose:
#   Create (*)starkit and application binaries for unix and windows.
#	This makefile might be used to "cross compile" binaries
#	for any target OS.
#
# ------------------------------------------------------------------------
# PLEASE NOTE:
#   There are various tclkit binaries available, but not all are
#   fully compatible with Tkhtml 3.0 loadable dll ?!
#   Tkhtml 3.0 is downloaded from ActiveState, which provides
#   binaries for Win as well as OSX !
#
#   If there is a version conflict:
#     as a side effect (without waring), images are not shown in the
#     html window !!!
#
#   does not work:
#     kbskit: WindowsNT64_kbsvq8.6-gui.exe
#     ActiceState: base-tk8.6-thread-win32-ix86.exe
#
#   does work:
#     TclKit tclkit-8.6.3-win32-ix86.exe
# -------------------------------------------------------------------------
# -------------------------------------------------------------------------


# main entry point:
set THIS_APP    [file normalize [file dirname [info script]]]
set TCLKIT_DIR  [file join $THIS_APP "../tclkit"]
set INSTALL_DIR [file join $THIS_APP "../Apps"]


# unix / win / osx ...
set OSX_KIT [file join $TCLKIT_DIR "Darwin_kbsvq8.6-gui"]
set WIN_KIT [file join $TCLKIT_DIR "WindowsNT_kbsvq8.6-gui.exe"]
set SDXUTIL [file join $TCLKIT_DIR "sdx.exe"]
set UPXUTIL [file join $TCLKIT_DIR "upx.exe"]


# -------------------------------------
# *edit* to reflect your build platform
# you are currently running on:
# -------------------------------------

if {$::tcl_version >= 8.4} {
	set plat [tk windowingsystem]
} else {
	set plat $::tcl_platform(platform)
}

switch -exact $plat {
	"aqua" {
		set TCL_KIT $OSX_KIT
	}
	"win32" -
	"windows" {

		# show the console window
		package require Tk
		wm withdraw .
		console show
		console eval {wm protocol . WM_DELETE_WINDOW {exit 0}}

		set TCL_KIT $WIN_KIT
	}
	"x11" -
	default {
		tk_messageBox \
		-title "Message:" \
		-message \
"Sorry, your platform: $plat you are currently working on
is not supported (due to missing kit file binaries)!
Please edit [info script] and try again!" \
		-icon info
		exit 0
	}
}


# -------------------------------------
set APPNAME "Hello_World"
set VER "0.1"
set APP "${APPNAME}${VER}"
set VFS "Release_${VER}"
# -------------------------------------


# default operation mode
if {$argc == 0} {
	set argv "win"
}

set t0 [clock milliseconds]

proc build_win {} {

	global APP VFS SDXUTIL TCL_KIT UPXUTIL

	puts "Creating windows executable: ${APP}.exe ..."; update

	file delete -force ${APP}.exe
	exec $SDXUTIL wrap ${APP}.exe -vfs $VFS -runtime $TCL_KIT
	
	# compressing the exe...
	puts "Running UPX to compress executable ..."; update
	exec $UPXUTIL ${APP}.exe
}

proc build_osx {} {

	global APPNAME APP VFS SDXUTIL OSX_KIT

	puts "Creating osx executable: ${APP}.osx ..."; update

	file delete -force ${APP}.osx
	exec $SDXUTIL wrap ${APP}.osx -vfs $VFS -runtime $OSX_KIT
	
	puts "Establishing required bundle directory structure ..."
	file mkdir ${APP}.app
	cd ${APP}.app
	file mkdir Contents
	cd Contents
		
	puts "Creating application specific Info.plist ..."
	file copy -force ../../macbundle/Info.plist ./Info.plist

	file mkdir MacOS
	file copy -force ../../${APP}.osx ./MacOS
	file mkdir Resources
	file copy -force ../../macbundle/${APPNAME}.icns ./Resources
	cd ../..
	file delete -force ${APP}.osx
}

proc build_sdx {} {

	global TCL_KIT SDXUTIL TCL_KIT

	puts "Creating windows executable: sdx.exe ..."; update

	file delete -force sdx.exe
	exec $TCL_KIT $SDXUTIL wrap sdx.exe -vfs $SDXUTIL -runtime $TCL_KIT
}

set t0 [clock milliseconds]

switch -- $argv {
	"win" {	build_win }
	"osx" {	build_osx }
	"sdx" {	build_sdx }

	"all" {
		build_win
		build_osx
	}
	
	default {
		puts "Unknown or missing argument!"
	}
}

puts "Done - [expr ( [clock milliseconds] - $t0 ) / 1000.0] sec."

# -------------------------------------
# EOF
